@@ -5,29 +5,20 @@ use std::path;
 use serde::{Deserialize, Serialize};
 
 use crate::build_system::FlatpakBuildSystem;
-use crate::source::FlatpakSource;
+use crate::format::FlatpakManifestFormat;
+use crate::source::{FlatpakSourceItem, ARCHIVE, GIT, PATCH};
 
 #[derive(Clone)]
 #[derive(Deserialize)]
 #[derive(Serialize)]
 #[derive(Debug)]
 #[derive(Hash)]
-#[serde(rename_all = "kebab-case")]
 #[serde(untagged)]
-/// Each module item can be either a path to a module description file,
-/// or an inline module description.
-pub enum FlatpakModule {
+/// Items in a module list can be either paths to external module manifests, or inline descriptions
+/// of flatpak modules.
+pub enum FlatpakModuleItem {
     Path(String),
-    Description(FlatpakModuleDescription),
-}
-impl FlatpakModule {
-    pub fn get_all_repos_urls(&self) -> Vec<String> {
-        if let FlatpakModule::Description(module_description) = self {
-            return module_description.get_all_urls();
-        } else {
-            return vec![];
-        }
-    }
+    Description(FlatpakModule),
 }
 
 #[derive(Clone)]
@@ -43,7 +34,10 @@ impl FlatpakModule {
 /// building.
 ///
 /// Modules can be nested, in order to turn related modules on and off with a single key.
-pub struct FlatpakModuleDescription {
+pub struct FlatpakModule {
+    #[serde(skip_serializing)]
+    pub format: FlatpakManifestFormat,
+
     /// The name of the module, used in e.g. build logs. The name is also
     /// used for constructing filenames and commandline arguments,
     /// therefore using spaces or '/' in this string is a bad idea.
@@ -57,7 +51,7 @@ pub struct FlatpakModuleDescription {
     /// String members in the array are interpreted as the name of a separate
     /// json or yaml file that contains sources. See below for details.
     #[serde(skip_serializing_if = "Vec::is_empty")]
-    pub sources: Vec<FlatpakSource>,
+    pub sources: Vec<FlatpakSourceItem>,
 
     /// An array of options that will be passed to configure
     #[serde(skip_serializing_if = "Vec::is_empty")]
@@ -102,7 +96,7 @@ pub struct FlatpakModuleDescription {
     /// Build system to use.
     #[serde(deserialize_with = "crate::build_system::deserialize_from_string")]
     #[serde(serialize_with = "crate::build_system::serialize_to_string")]
-    pub buildsystem: FlatpakBuildSystem,
+    pub buildsystem: Option<FlatpakBuildSystem>,
 
     /// Use a build directory that is separate from the source directory
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -170,14 +164,14 @@ pub struct FlatpakModuleDescription {
     /// String members in the array are interpreted as names of a separate json or
     /// yaml file that contains a module.
     #[serde(skip_serializing_if = "Vec::is_empty")]
-    pub modules: Vec<FlatpakModule>,
+    pub modules: Vec<FlatpakModuleItem>,
 }
-impl FlatpakModuleDescription {
+impl FlatpakModule {
     pub fn uses_external_data_checker(&self) -> bool {
         for source in &self.sources {
             let source_description = match source {
-                FlatpakSource::Description(d) => d,
-                FlatpakSource::Path(_) => continue,
+                FlatpakSourceItem::Description(d) => d,
+                FlatpakSourceItem::Path(_) => continue,
             };
             if source_description.x_checker_data.is_some() {
                 return true;
@@ -189,7 +183,7 @@ impl FlatpakModuleDescription {
     pub fn get_all_mirror_urls(&self) -> Vec<String> {
         let mut all_urls = vec![];
         for module in &self.modules {
-            if let FlatpakModule::Description(module_description) = module {
+            if let FlatpakModuleItem::Description(module_description) = module {
                 all_urls.append(&mut module_description.get_all_mirror_urls());
             }
         }
@@ -205,14 +199,17 @@ impl FlatpakModuleDescription {
         if self.cmake.unwrap_or(false) {
             return Some(crate::build_system::CMAKE.to_string());
         }
-        return Some(self.buildsystem.to_string());
+        if let Some(buildsystem) = &self.buildsystem {
+            return Some(buildsystem.to_string());
+        }
+        None
     }
 
     pub fn is_patched(&self) -> bool {
         for source in &self.sources {
-            if let FlatpakSource::Description(sd) = source {
+            if let FlatpakSourceItem::Description(sd) = source {
                 if let Some(t) = &sd.r#type {
-                    if t == "patch" {
+                    if t == PATCH {
                         return true;
                     }
                 }
@@ -221,52 +218,51 @@ impl FlatpakModuleDescription {
         false
     }
 
-    pub fn load_from_file(path: String) -> Result<FlatpakModuleDescription, String> {
+    pub fn load_from_file(path: String) -> Result<FlatpakModule, String> {
         let file_path = path::Path::new(&path);
         if !file_path.is_file() {
             return Err(format!("{} is not a file.", path));
         }
 
-        if FlatpakModuleDescription::file_path_matches(&file_path.to_str().unwrap()) {
-            let module_content = match fs::read_to_string(file_path) {
-                Ok(content) => content,
-                Err(e) => {
-                    return Err(format!(
-                        "Could not read file {}: {}!",
-                        file_path.to_str().unwrap(),
-                        e
-                    ));
-                }
-            };
-            let module = match FlatpakModuleDescription::parse(&path, &module_content) {
-                Ok(m) => m,
-                Err(e) => return Err(format!("Failed to parse Flatpak module at {}: {}", path, e)),
-            };
-            return Ok(module);
-        } else {
-            return Err(format!("{} is not a Flatpak module.", path));
+        let manifest_format = match FlatpakManifestFormat::from_path(&path) {
+            Some(f) => f,
+            None => return Err(format!("{} is not a Flatpak module manifest.", path)),
+        };
+
+        let manifest_content = match fs::read_to_string(file_path) {
+            Ok(content) => content,
+            Err(e) => {
+                return Err(format!("Could not read module manifest at {}: {}", &path, e));
+            }
+        };
+        match FlatpakModule::parse(manifest_format, &manifest_content) {
+            Ok(m) => Ok(m),
+            Err(e) => Err(format!(
+                "Failed to parse Flatpak module manifest at {}: {}",
+                path, e
+            )),
         }
     }
 
-    pub fn parse(module_path: &str, module_content: &str) -> Result<FlatpakModuleDescription, String> {
-        let mut flatpak_module: FlatpakModuleDescription = FlatpakModuleDescription::default();
-
-        if module_path.to_lowercase().ends_with("yaml") || module_path.to_lowercase().ends_with("yml") {
-            flatpak_module = match serde_yaml::from_str(&module_content) {
+    pub fn parse(format: FlatpakManifestFormat, manifest_content: &str) -> Result<FlatpakModule, String> {
+        let mut flatpak_module: FlatpakModule = match &format {
+            FlatpakManifestFormat::YAML => match serde_yaml::from_str(&manifest_content) {
                 Ok(m) => m,
                 Err(e) => {
-                    return Err(format!("Failed to parse the Flatpak manifest: {}.", e));
+                    return Err(format!("Failed to parse the Flatpak module manifest: {}.", e));
                 }
-            };
-        } else if module_path.to_lowercase().ends_with("json") {
-            let json_content_without_comments = crate::utils::remove_comments_from_json(module_content);
-            flatpak_module = match serde_json::from_str(&json_content_without_comments) {
-                Ok(m) => m,
-                Err(e) => {
-                    return Err(format!("Failed to parse the Flatpak manifest: {}.", e));
+            },
+            FlatpakManifestFormat::JSON => {
+                let json_content_without_comments = crate::utils::remove_comments_from_json(manifest_content);
+                match serde_json::from_str(&json_content_without_comments) {
+                    Ok(m) => m,
+                    Err(e) => {
+                        return Err(format!("Failed to parse the Flatpak module manifest: {}.", e));
+                    }
                 }
-            };
-        }
+            }
+        };
+        flatpak_module.format = format;
 
         if flatpak_module.name.is_empty() {
             return Err("Required top-level field name is missing from Flatpak module.".to_string());
@@ -276,8 +272,8 @@ impl FlatpakModuleDescription {
         }
         for source in &flatpak_module.sources {
             let source_path = match source {
-                FlatpakSource::Description(_) => continue,
-                FlatpakSource::Path(p) => p,
+                FlatpakSourceItem::Description(_) => continue,
+                FlatpakSourceItem::Path(p) => p,
             };
             // The string elements of the source array should only be FS paths, not
             // URLs or anything else.
@@ -305,7 +301,7 @@ impl FlatpakModuleDescription {
     pub fn get_all_urls(&self) -> Vec<String> {
         let mut all_urls = vec![];
         for module in &self.modules {
-            if let FlatpakModule::Description(module_description) = module {
+            if let FlatpakModuleItem::Description(module_description) = module {
                 all_urls.append(&mut module_description.get_all_urls());
             }
         }
@@ -320,13 +316,13 @@ impl FlatpakModuleDescription {
     pub fn get_all_archive_urls(&self) -> Vec<String> {
         let mut all_archive_urls = vec![];
         for source in &self.sources {
-            if source.get_type_name() != "archive" {
+            if source.get_type_name() != ARCHIVE {
                 continue;
             }
 
             let source_description = match &source {
-                FlatpakSource::Path(_) => continue,
-                FlatpakSource::Description(d) => d,
+                FlatpakSourceItem::Path(_) => continue,
+                FlatpakSourceItem::Description(d) => d,
             };
 
             let archive_url = match &source_description.url {
@@ -341,13 +337,13 @@ impl FlatpakModuleDescription {
     pub fn get_all_git_urls(&self) -> Vec<String> {
         let mut all_git_urls = vec![];
         for source in &self.sources {
-            if source.get_type_name() != "git" {
+            if source.get_type_name() != GIT {
                 continue;
             }
 
             let source_description = match &source {
-                FlatpakSource::Path(_) => continue,
-                FlatpakSource::Description(d) => d,
+                FlatpakSourceItem::Path(_) => continue,
+                FlatpakSourceItem::Description(d) => d,
             };
 
             let git_url = match &source_description.url {
@@ -363,7 +359,7 @@ impl FlatpakModuleDescription {
     pub fn get_max_depth(&self) -> i32 {
         let mut max_depth: i32 = 0;
         for module in &self.modules {
-            if let FlatpakModule::Description(module_description) = module {
+            if let FlatpakModuleItem::Description(module_description) = module {
                 let module_depth = module_description.get_max_depth();
                 if module_depth > max_depth {
                     max_depth = module_depth;
@@ -390,9 +386,9 @@ impl FlatpakModuleDescription {
         }
     }
 
-    pub fn get_all_modules_recursively(&self) -> Vec<&FlatpakModule> {
-        let mut all_modules: Vec<&FlatpakModule> = vec![];
-        let mut next_modules: Vec<&FlatpakModule> = vec![];
+    pub fn get_all_modules_recursively(&self) -> Vec<&FlatpakModuleItem> {
+        let mut all_modules: Vec<&FlatpakModuleItem> = vec![];
+        let mut next_modules: Vec<&FlatpakModuleItem> = vec![];
         for module in &self.modules {
             next_modules.push(module);
         }
@@ -401,8 +397,8 @@ impl FlatpakModuleDescription {
             all_modules.push(module);
 
             let module = match module {
-                FlatpakModule::Description(d) => d,
-                FlatpakModule::Path(_) => continue,
+                FlatpakModuleItem::Description(d) => d,
+                FlatpakModuleItem::Path(_) => continue,
             };
             for next_module in &module.modules {
                 next_modules.push(next_module);
@@ -410,6 +406,22 @@ impl FlatpakModuleDescription {
         }
         all_modules
     }
+
+    /// A module is composite if it links to multiple software projects.
+    /// This is determined by the type of the sources contained in the module.
+    pub fn is_composite(&self) -> bool {
+        let mut code_sources_count = 0;
+        for source in &self.sources {
+            let source_type_name = source.get_type_name();
+            if crate::source::CODE_TYPES.contains(&source_type_name) {
+                code_sources_count += 1;
+            }
+            if code_sources_count > 1 {
+                return true;
+            }
+        }
+        false
+    }
 }
 
 #[derive(Clone)]
@@ -544,9 +556,7 @@ mod tests {
 
     #[test]
     pub fn test_parse_extra_data() {
-        match FlatpakModuleDescription::parse(
-            "module.yaml",
-            r###"
+        let module_manifest = r###"
             name: wps
             buildsystem: simple
             build-commands:
@@ -582,25 +592,42 @@ mod tests {
                   url: https://linux.wps.com/js/meta.js
                   version-pattern: version\s*=\s*"([\d.-]+)"
                   url-pattern: download_link_deb\s*=\s*"(http[s]?://[\w\d$-_@.&+]+)"
-            "###,
-        ) {
-            Err(e) => panic!(e),
-            Ok(module) => {
-                assert_eq!(module.name, "wps");
-            }
-        }
+        "###;
+        let module = FlatpakModule::parse(FlatpakManifestFormat::YAML, module_manifest).unwrap();
+        assert_eq!(module.name, "wps");
     }
 
     #[test]
     pub fn test_parse_helm_files() {
-        assert!(FlatpakModuleDescription::parse(
-            "module.yaml",
-            r###"
+        let helm_file = r###"
             name: wps
             sources:
               - "https://github.com/user/repo"
-            "###,
-        )
-        .is_err())
+        "###;
+        assert!(FlatpakModule::parse(FlatpakManifestFormat::YAML, helm_file,).is_err())
+    }
+
+    #[test]
+    pub fn test_parse_no_buildsystem() {
+        let module_manifest = r###"
+            name: dbus-glib
+            sources:
+              - type: archive
+                url: "https://dbus.freedesktop.org/releases/dbus-glib/dbus-glib-0.110.tar.gz"
+                sha256: 7ce4760cf66c69148f6bd6c92feaabb8812dee30846b24cd0f7395c436d7e825
+            config-opts:
+              - "--disable-static"
+              - "--disable-gtk-doc"
+            cleanup:
+              - "*.la"
+              - /bin
+              - /etc
+              - /include
+              - /libexec
+              - /share/gtk-doc
+              - /share/man
+        "###;
+        let flatpak_application = FlatpakModule::parse(FlatpakManifestFormat::YAML, module_manifest).unwrap();
+        assert!(flatpak_application.buildsystem.is_none())
     }
 }
